var get = require('lodash.get');
var jsonSchemaToYup = require('json-schema-to-yup');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var get__default = /*#__PURE__*/_interopDefaultLegacy(get);

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;
  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
  return arr2;
}
function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);
  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }
  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

// Desc: Configs for the translation that is used in the validation messages
var translatesConfig = {
  required: 'validation.required',
  min: 'validation.min',
  max: 'validation.max',
  maxLength: 'validation.max_length',
  minLength: 'validation.min_length',
  pattern: 'validation.pattern',
  format: 'validation.format',
  email: 'validation.email'
};
// Desc: Enum for the validation error types
var EValidationTypes;
(function (EValidationTypes) {
  EValidationTypes["required"] = "required";
  EValidationTypes["min"] = "min";
  EValidationTypes["max"] = "max";
  EValidationTypes["maxLength"] = "maxLength";
  EValidationTypes["minLength"] = "minLength";
  EValidationTypes["pattern"] = "pattern";
  EValidationTypes["format"] = "format";
})(EValidationTypes || (EValidationTypes = {}));

// @ts-ignore
/**
 * @param formDataChildren - array of form fields
 * @param acc - accumulator, used for recursion
 * @returns {[key: string]: any} - object with the initial values for the form
 * @description getInitialValues function returns an object with the initial values for the form
 * based on the initial values provided in the json schema
 */

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }
  if (result && result.then) {
    return result.then(void 0, recover);
  }
  return result;
} /**
   * @param schema - yup schema
   * @param fieldName - field name to get the rule value from
   * @param ruleName - rule name to get the value from
   * @returns {any} - rule value
   * @description getRuleValueFromSchema function returns the value of a rule from a yup schema
   * based on the field name and rule name. This value is needed for passing meta to translations.
   * @example getRuleValueFromSchema(schema, 'email', 'minLength') => 6
   */
var getInitialValues = function getInitialValues(formDataChildren, acc) {
  if (acc === void 0) {
    acc = {};
  }
  return formDataChildren.reduce(function (acc, item) {
    if (isFormGroup(item)) {
      return getInitialValues(item.children, acc);
    }
    acc[item.fieldName] = item.initialValue === null ? undefined : item.initialValue;
    return acc;
  }, acc);
};
var getRuleValueFromSchema = function getRuleValueFromSchema(schema, fieldName, ruleName) {
  var describedSchema = schema.describe();
  var rules = get__default["default"](describedSchema, "fields." + fieldName + ".tests", []);
  var rule = rules.find(function (rule) {
    return rule.name === ruleName;
  });
  return get__default["default"](rule, "params." + ruleName, undefined);
};
/**
 * @param type - validation type
 * @param value - value to be validated
 * @param t - translation function(optional)
 * @param validationParam - validation param(meta) to be passed to the translation function(optional) or error object
 * @returns {any} - error object or function, or translated string
 * @description getFieldError function returns an error object or function, or translated string based on the validation type
 */
var getFieldError = function getFieldError(type, value, t, validationParam) {
  var translationKey = get__default["default"](translatesConfig, "" + type);
  switch (type) {
    case EValidationTypes.required:
      return t ? t(translationKey) : createErrorObject(translationKey);
    case EValidationTypes.min:
      return t ? function (_ref) {
        var min = _ref.min;
        return t(translationKey, {
          param: min
        });
      } : createErrorObject(translationKey, validationParam);
    case EValidationTypes.max:
      return t ? function (_ref2) {
        var max = _ref2.max;
        return t(translationKey, {
          param: max
        });
      } : createErrorObject(translationKey, validationParam);
    case EValidationTypes.maxLength:
      return t ? function (_ref3) {
        var maxLength = _ref3.maxLength;
        return t(translationKey, {
          param: maxLength
        });
      } : createErrorObject(translationKey, validationParam);
    case EValidationTypes.minLength:
      return t ? function (_ref4) {
        var minLength = _ref4.minLength;
        return t(translationKey, {
          param: minLength
        });
      } : createErrorObject(translationKey, validationParam);
    case EValidationTypes.pattern:
      return t ? t(translationKey) : createErrorObject(translationKey, validationParam);
    case EValidationTypes.format:
      if (value) {
        translationKey = get__default["default"](translatesConfig, "" + type) ? get__default["default"](translatesConfig, "" + type) : translationKey;
      }
      return t ? t(translationKey) : createErrorObject(translationKey, validationParam);
    default:
      return null;
  }
};
/**
 * @param form - form json schema
 * @param formValues - form values to be validated
 * @returns {IValidationErrors} - object with the validation errors
 * @description makeCrossFieldValidation function makes custom cross-field validation
 * and returns an object with the validation errors
 */
var makeCrossFieldValidation = function makeCrossFieldValidation(form, formValues) {
  var errors = {};
  var validationObject = form.validationObject;
  try {
    Object.keys(validationObject.crossFieldValidations).forEach(function (fieldName) {
      var rules = validationObject.crossFieldValidations[fieldName];
      var fieldErrors = {
        property: fieldName,
        constraints: {}
      };
      rules.forEach(function (rule, id) {
        var errorKey = rule.errorKey,
          testCondition = rule.testCondition,
          meta = rule.meta;
        if (functionFromString(testCondition)(formValues)) {
          fieldErrors.constraints[id] = createErrorObject(errorKey, meta);
        }
      });
      if (Object.keys(fieldErrors.constraints).length > 0) {
        errors[fieldName] = fieldErrors;
      }
    });
    return errors;
  } catch (e) {
    console.warn('Error in makeCrossFieldValidation: ', e);
    return {};
  }
};
/**
 * @param jsonSchema - json validation schema
 * @param t - translation function(optional)
 * @returns {AnyObjectSchema} - yup validation schema
 * @description buildYupSchema function builds a yup validation schema based on the json validation schema
 */
var buildYupSchema = function buildYupSchema(jsonSchema, t) {
  return jsonSchemaToYup.buildYup(jsonSchema, t ? {
    errMessages: getErrorMessages(jsonSchema, t)
  } : undefined);
};
/**
 * @param form {IForm} - form json schema
 * @param valuesToValidate {any} - form values to be validated
 * @param skipCrossField {boolean} - skip cross-field validation(optional)
 * @returns {Promise<unknown>} - object with the validation errors or error that has occurred during the validation
 * @description validateValues function validates the form values based on the json form object
 */
var validateValues = function validateValues(form, valuesToValidate, skipCrossField) {
  if (skipCrossField === void 0) {
    skipCrossField = false;
  }
  try {
    var _temp2 = function _temp2(_result) {
      return _exit ? _result : {
        errors: Object.values(errors)
      };
    };
    var _exit;
    var errors = {};
    var _temp = _catch(function () {
      var validationSchema = buildYupSchema(form.validationObject);
      if (!skipCrossField) {
        errors = makeCrossFieldValidation(form, valuesToValidate);
      }
      return Promise.resolve(validationSchema.validate(valuesToValidate, {
        abortEarly: false
      })).then(function () {});
    }, function (e) {
      if (get__default["default"](e, 'name') === 'ValidationError') {
        errors = formatYupValidationErrors(e, errors);
      } else {
        _exit = 1;
        return e;
      }
    });
    return Promise.resolve(_temp && _temp.then ? _temp.then(_temp2) : _temp2(_temp));
  } catch (e) {
    return Promise.reject(e);
  }
};

/**
 * @param jsonSchema - jsonSchema validation object
 * @param t - i18n translate function
 * @returns {any}
 * @description getErrorMessages function returns an object with the error messages for each field in the form based on the jsonSchema
 */
var getErrorMessages = function getErrorMessages(jsonSchema, t) {
  var errMessages = {};
  Object.keys(jsonSchema.properties).forEach(function (property) {
    errMessages[property] = {};
    Object.keys(jsonSchema.properties[property]).forEach(function (key) {
      var value = jsonSchema.properties[property][key];
      errMessages[property][key] = getFieldError(key, value, t);
      if (Object.keys(jsonSchema).includes(key)) {
        errMessages[property].required = getFieldError('required', null, t);
      }
    });
  });
  return errMessages;
};
/**
 * @param translateKey
 * @param param
 * @returns {IValidationError}
 * @description createErrorObject function returns an object with the error message
 * for a field in the form in a proper format
 * @example createErrorObject('required', { param: 'email' }) => { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: { param: 'email' } }
 */
var createErrorObject = function createErrorObject(translateKey, param) {
  return {
    translationKey: translateKey,
    fallbackLanguageTranslation: translateKey,
    meta: param ? {
      param: param
    } : {}
  };
};
function isFormGroup(toBeDetermined) {
  return !!toBeDetermined.children;
}
/**
 * @param stringifiedFunction
 * @returns {Function}
 * @description functionFromString returns a function from a stringified function
 * @example functionFromString('/Function((values) => {return values.medicalConditions.toString() === 'true'})/')
 * => (values) => values.medicalConditions.toString() === 'true'
 */
var functionFromString = function functionFromString(stringifiedFunction) {
  try {
    var substr = stringifiedFunction.substring(10, stringifiedFunction.length - 2);
    return (0, eval)('(' + substr + ')');
  } catch (e) {
    console.warn('Error parsing function in functionFromString: ', e);
    return function () {
      return false;
    };
  }
};
/**
 * @param validationError - yup validation error
 * @param initialErrors - initial errors object to be merged with the yup validation errors
 * @returns {IValidationErrors} - returns an object with the errors for each field in the form
 * @description formatYupValidationErrors function returns an object with the errors for each field in the form
 * based on the yup validation error formatting to be used in the GenericForm component
 * @example formatYupValidationErrors(yupValidationError, { email: { property: 'email', constraints: { required: { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: {} } } } })
 */
var formatYupValidationErrors = function formatYupValidationErrors(validationError, initialErrors) {
  if (initialErrors === void 0) {
    initialErrors = {};
  }
  var errors = _extends({}, initialErrors);
  for (var _iterator = _createForOfIteratorHelperLoose(validationError.inner), _step; !(_step = _iterator()).done;) {
    var propertyValidationError = _step.value;
    if (!propertyValidationError.type) {
      continue;
    }
    if (!get__default["default"](errors, "" + propertyValidationError.path)) {
      errors[propertyValidationError.path] = {
        property: propertyValidationError.path,
        constraints: {}
      };
    }
    errors[propertyValidationError.path].constraints[propertyValidationError.type] = _extends({}, getFieldError(propertyValidationError.type, get__default["default"](propertyValidationError, 'path', null), undefined, get__default["default"](propertyValidationError.params, "" + propertyValidationError.type)), {
      fallbackLanguageTranslation: propertyValidationError.message
    });
  }
  return errors;
};

exports.buildYupSchema = buildYupSchema;
exports.createErrorObject = createErrorObject;
exports.formatYupValidationErrors = formatYupValidationErrors;
exports.functionFromString = functionFromString;
exports.getErrorMessages = getErrorMessages;
exports.getFieldError = getFieldError;
exports.getInitialValues = getInitialValues;
exports.getRuleValueFromSchema = getRuleValueFromSchema;
exports.isFormGroup = isFormGroup;
exports.makeCrossFieldValidation = makeCrossFieldValidation;
exports.validateValues = validateValues;
//# sourceMappingURL=index.js.map
